! Module: tablesort_mod.f90 
! Author: Peter Videler <peter@videler.nl>
! 
! (c) 2000 European Space Agency 
!
! Description: Sort whole Table on column.
!! 
! Comments:  - Good candidate for implementation with DAL iterators 
!            - uses SLATEC qs2i1r for sorting
! 
! Modifications: 
! 
! $Id: tablesort_mod.f90,v 1.2 2000/02/23 12:56:12 pvideler Exp $

module tablesort_mod
  use dal         
  use errorHandling

  !......................................................................
  implicit none 
  private    ! change default to private
  public  ::  tableSort
  !......................................................................

contains 

  ! ====================================================================
  subroutine tableSort (tbl, sortColumn, sortorder) 
    ! ====================================================================
    ! sort ALL columns of table according to int32 column columnName
    ! sortorder = +1 increasing
    ! sortorder = -1 decreasing

    type (tableT),     intent(inout)   :: tbl  
    type (ColumnT),    intent(in)      :: sortColumn  
    integer, optional, intent(in)      :: sortorder

    type (ColumnT)                    :: sortIndexCol, col
    integer                           :: i, sz, sortorder_
    character(len=*),    parameter    :: fn='tableSort: '

    integer, pointer, dimension(:)    :: sortIndex
    real,    pointer, dimension(:)    :: dummyRealArray

    integer(kind=int32), pointer, dimension(:) :: i32

    if (numberOfColumns(tbl) < 1) RETURN
    if (numberOfRows(tbl) < 2)    RETURN

    if (present(sortorder)) then
       sortorder_ = sortorder
    else
       sortorder_ = +1     ! default sort increasing order 
    endif

    sz = NumberOfRows(tbl) 
    if (NumberOfRows(sortColumn) /= sz ) call error (fn//'01', 'column sizes do not match')

    write(errstr,*) fn//'Sorting table ',trim(name(tbl)), &
                   ' on column ', trim(name(sortColumn))  
    call Message (AppMsg, Verbose) 

    ! make local copy because sortColumn could be in read only set and 
    ! qs2i1r also sorts the sortColumn

    call copyColumn(tbl, sortColumn, 'DUMMY')
    col = column(tbl, 'DUMMY')

    ! ### Can ONLY sort on int32 for the moment. ###     
    select case (dataType(col))
    case(integer32) 
       i32 =>  int32data(col)
    case default 
       call error (fn, 'datatype not implemented')
    end select
    
    sortIndexCol = addColumn(tbl, 'SORTINDEX', integer32, 'Sort index added by tableSort')
    sortIndex => int32data(sortIndexCol)
    sortIndex = (/(i, i=0, sz-1, 1)/)

    ! note slatec qs2i1r: 
    ! - can't sort zero length arrays.
    ! - needs dummyArray (real)
    ! - sorts input i32 array as well rather then returning only a sortindex vector

    allocate(dummyRealArray(0:sz-1))
    dummyRealArray = real(sortIndex)		! needed for slatec routine 
    call qs2i1r(i32, sortIndex, dummyRealArray, sz, sortorder_)
    deallocate (dummyRealArray)

    call release(col)             ! need to release column before delete
    call deleteColumn(tbl, 'DUMMY')

    ! Sort all columns but the last one (sortindex) in order given by sortindex

    do i = 0, numberOfColumns(tbl) - 2
       col = column(tbl, i, modify)
       call columnSort(col, sortIndex)
    enddo

    call release(sortIndexCol)

  end subroutine tableSort

!---------------------------------------
  subroutine columnSort (col, sortIndex) 
    !---------------------------------------

    type (ColumnT), intent(inout)            :: col
    integer       , intent(in), dimension(:) :: sortIndex
    character(len=*),    parameter    :: fn='columnSort: '

    integer(kind=int8) , pointer, dimension(:) :: i8
    integer(kind=int16), pointer, dimension(:) :: i16
    integer(kind=int32), pointer, dimension(:) :: i32
    real(kind=single)  , pointer, dimension(:) :: r32
    real(kind=double)  , pointer, dimension(:) :: r64

    if (numberOfRows(col) /= size(sortIndex)) call error (fn, 'size mismatch')

    select case (dataType(col))     
    case(integer8)  ; i8  =>   int8data(col) ; i8  = i8 (sortIndex)
    case(integer16) ; i16 =>  int16data(col) ; i16 = i16(sortIndex)
    case(uinteger16); i16 => uint16data(col) ; i16 = i16(sortIndex)
    case(integer32) ; i32 =>  int32data(col) ; i32 = i32(sortIndex)
    case(uinteger32); i32 => uint32data(col) ; i32 = i32(sortIndex)
    case(real32)    ; r32 => real32data(col) ; r32 = r32(sortIndex)
    case(real64)    ; r64 => real64data(col) ; r64 = r64(sortIndex)
    case default    ; call error (fn, 'datatype not implemented')
    end select

  end subroutine

  end module





















