module matrix4_mod

!  $Id: matrix4_mod.f90,v 1.1 1999/04/01 12:22:33 pvideler Exp $
!+
! matrix4_mod.f90	(c) 1996 European Space Agency
!
! AUTHOR:	Marco Beijersbergen
! VERSION:	1.12
! DESCRIPTION:	Function to manipulate 4x4 matrices for geometric
!		transformations
!--
! COMMENTS:	type 'double' version. type single to be added when needed. 
!
! MODIFICATIONS:
!    VERSION	DATE	AUTHOR	DESCRIPTION OF MOD.
!     1.11	13-8-96	MBe	changed sign convention in Rotation
!     1.12	8-1-99	PVi	Taken from SciSim. 'kr' replaced by 'double', 'ki' removed
!                                'use types_mod' replaced by 'use types'
!                               added public/private
!+

  use types       ! SAS data types

  implicit none

 !......................................................................
  private
  public  ::              &
      matrix4Unity,       &        
      matrix4Translation, &
      matrix4Rotation,    &      
      matrix4Inverse
 !......................................................................

contains

  function matrix4Unity() result(m)
    real(double), dimension(4,4) :: m

    integer i

    m = 0d0
    do i = 1, size(m(:,1))
      m(i,i) = 1d0
    end do

  end function matrix4Unity

  function matrix4Translation(v) result(m)
    real(double), dimension(3), intent(in) :: v
    real(double), dimension(4,4) :: m

    m = matrix4Unity()
    m(1:3,4) = v

  end function matrix4Translation

! Calculate a matrix for rotation around a given axis.
! See Foley et al, 'Computer Graphics: Principles and Practice', Addison-Wesley,
! Reading, Massachusetts, 2nd edition (1993).
  function matrix4Rotation(v,radians) result(m)
    real(double), dimension(3), intent(in) :: v
    real(double), intent(in) :: radians
    real(double), dimension(4,4) :: m

    real(double) cosa, sina, one_min_cosa
    real(double) vxy_one_min_cosa, vyz_one_min_cosa, vxz_one_min_cosa
    real(double), dimension(3) :: v2

    m = matrix4Unity()

    cosa = cos(radians)
    sina = sin(radians)
    v2 = v*v

    one_min_cosa = 1d0 - cosa
    vxy_one_min_cosa = v(1)*v(2)*one_min_cosa
    vxz_one_min_cosa = v(1)*v(3)*one_min_cosa
    vyz_one_min_cosa = v(2)*v(3)*one_min_cosa

    m(1,1) = v2(1) + cosa*(1d0 - v2(1));
    m(1,2) = vxy_one_min_cosa - v(3)*sina
    m(1,3) = vxz_one_min_cosa + v(2)*sina
    m(2,1) = vxy_one_min_cosa + v(3)*sina
    m(2,2) = v2(2) + cosa*(1d0 - v2(2))
    m(2,3) = vyz_one_min_cosa - v(1)*sina
    m(3,1) = vxz_one_min_cosa - v(2)*sina
    m(3,2) = vyz_one_min_cosa + v(1)*sina
    m(3,3) = v2(3) + cosa*(1d0 - v2(3))

  end function matrix4Rotation

! Invert a 4x4 matrix. 
  function matrix4Inverse(m) result(mInv)
    real(double), dimension(4,4), intent(in) :: m
    real(double), dimension(4,4) :: mInv

    integer   :: i,j,k
    real(double) mulby, divby
    real(double), dimension(4,4) :: x

    x = m
    mInv = matrix4Unity()

    do i=1,4

      if (x(i,i) /= 1d0) then
        divby = x(i,i)
        do j=1,4
          mInv(i,j) = mInv(i,j) / divby
          x(i,j) = x(i,j) / divby
        end do
      end if

      do j=1,4
        if (j /= i) then
          if (x(j,i) /= 0d0) then
            mulby = x(j,i)
            do k=1,4
              x(j,k) = x(j,k) - mulby * x(i,k)
              mInv(j,k) = mInv(j,k) - mulby * mInv(i,k)
            end do
          end if
        end if
      end do

    end do

  end function matrix4Inverse

end module matrix4_mod















