! Module: notNear.f90 
! Author: P.Videler
! Version: 1.0
! 
! (c) 1997 European Space Agency 
!
! Description: compare  integer or reals (scalars and vectors) to see if their
!              relative difference is smaller than a given value.
! 
! Comments: 
! 
! Modifications: 
! 
! $Id: notneartest.f90,v 1.1 1999/04/01 12:22:35 pvideler Exp $
! 
module notNearTest

 use types         ! SAS data types
 use errorHandling

 implicit none 

 !......................................................................
 private    ! change default to private
 public  ::         &
      notNear,      &        
      notNear_epsilon

 !......................................................................

! test for not equal (near for reals)
 interface notNear
    module procedure neq_int16, neq_int32, neq_int16_array, neq_int32_array
    module procedure neq_single, neq_double, neq_single_array, neq_double_array
    module procedure neq_single_2darray
 end interface

! relative difference used in notNear test.
  real(kind=single)    :: notNear_epsilon = 1.e-6

contains 

 ! ====================================================================
 logical function neq_int16 (a,b)  result(bool)
  ! ====================================================================
  integer(int16),    intent(in)    :: a, b
  character(len=*),    parameter   :: fn='neq_int16: '

  if (a == b) then 
     bool=.false.
  else
     bool = .true.
     write(errstr,*) fn//'Integer comparison failed ',a,b
  endif

 end function neq_int16
 
 ! ====================================================================
 logical function neq_int32 (a,b)  result(bool)
  ! ====================================================================
  integer(int32),    intent(in)   :: a, b
  character(len=*),    parameter   :: fn='neq_int32: '

  if (a == b) then 
     bool=.false.
  else
     bool = .true.
     write(errstr,*) fn//'Integer comparison failed ',a,b
  endif

 end function neq_int32

 ! ====================================================================
 logical function neq_int16_array (a,b)  result(bool)
  ! ====================================================================
  integer(kind=int16),  intent(in) :: a(:)
  integer            ,  intent(in) :: b(:)
  character(len=*),     parameter  :: fn='neq_int16_array: '

  bool=.false.

  if (size(a) /= size(b)) then
      bool = .true.
      write(errstr,*) fn//'array dimensions do not match', size(a), size(b)
      return
  endif

  if (ANY (a /= b)) then 
     bool = .true.
     write(errstr,*) fn//'Integer comparison failed ',a,'is not equal to ',b
  endif

 end function 
 
 ! ====================================================================
 logical function neq_int32_array (a,b)  result(bool)
  ! ====================================================================
  integer(kind=int32),  intent(in) :: a(:)
  integer            ,  intent(in) :: b(:)
  character(len=*),     parameter  :: fn='neq_int32_array: '

  bool=.false.

  if (size(a) /= size(b)) then
      bool = .true.
      write(errstr,*) fn//'array dimensions do not match', size(a), size(b)
      return
  endif

  if (ANY (a /= b)) then 
     bool = .true.
     write(errstr,*) fn//'Integer comparison failed ',a,'is not equal to ',b
  endif

 end function 
  
 ! ====================================================================
 logical function neq_single (a,b)  result(bool)
  ! ====================================================================
  real(kind=single),    intent (in)   :: a, b
  character(len=*),     parameter   :: fn='neq_single: '

  bool=.false.

  if ( abs(a-b) > notNear_epsilon*abs(a) ) then 
     bool = .true.
     write(errstr,*) fn//'real (single) comparison failed ',a,b
  endif

 end function neq_single

 ! ====================================================================
 logical function neq_single_array (a,b)  result(bool)
  ! ====================================================================
  real(kind=single),    intent(in)    :: a(:), b(:)
  character(len=*),     parameter  :: fn='neq_single_array: '

  bool=.false.

  if (size(a) /= size(b)) then
      bool = .true.
      write(errstr,*) fn//'array dimensions do not match', size(a), size(b)
      return
  endif


  if ( ANY( abs(a-b) > notNear_epsilon*abs(a) ) ) then 
     bool = .true.
     write(errstr,*) fn//'real (single) comparison failed ',a,' does not match with ',b
  endif

 end function neq_single_array

 ! ====================================================================
 logical function neq_single_2darray (a,b)  result(bool)
  ! ====================================================================
  real(kind=single),    intent(in)    :: a(:,:), b(:,:)
  character(len=*),     parameter  :: fn='neq_single_2darray: '

  bool=.false.

  if (ANY(shape(a) /= shape(b))) then
      bool = .true.
      write(errstr,*) fn//'array shapes do not match', shape(a), shape(b)
      return
  endif


  if ( ANY( abs(a-b) > notNear_epsilon*abs(a) ) ) then 
     bool = .true.
     write(errstr,*) fn//'real (single) comparison failed ',a,' does not match with ',b
  endif

 end function

 ! ====================================================================
 logical function neq_double (a,b)  result(bool)
  ! ====================================================================
  real(kind=double),    intent (in)   :: a, b
  character(len=*),     parameter   :: fn='neq_double: '

  bool=.false.

  if ( abs(a-b) > notNear_epsilon*abs(a) ) then 
     bool = .true.
     write(errstr,*) fn//'real (double) comparison failed ',a,b
  endif

 end function neq_double

 ! ====================================================================
 logical function neq_double_array (a,b)  result(bool)
  ! ====================================================================
  real(kind=double),    intent(in)    :: a(:), b(:)
  character(len=*),     parameter  :: fn='neq_double_array: '

  bool=.false.

  if (size(a) /= size(b)) then
      bool = .true.
      write(errstr,*) fn//'array dimensions do not match', size(a), size(b)
      return
  endif


  if ( ANY( abs(a-b) > notNear_epsilon*abs(a) ) ) then 
     bool = .true.
     write(errstr,*) fn//'real (double) comparison failed ',a,b
  endif

 end function neq_double_array

 !***************************************************************************

 !---------------------------------------------------------------------------
 subroutine compare1_SingleSingle(v1, v2, fn)
  !---------------------------------------------------------------------------
  real(single),     intent(in) :: v1(:), v2(:)
  character(len=*), intent(in) :: fn

  if (size(v1) /= size(v2))  then 
     write(errstr,*) fn//': vector sizes not equal. Size1: ',size(v1),' :Size2 ',size(v2)
     call error ('histo')
  endif

 end subroutine compare1_SingleSingle

 !---------------------------------------------------------------------------
 subroutine compare2_SingleSingle(xx1, xx2, fn)
  !---------------------------------------------------------------------------
  real(single),  intent(in)    :: xx1(:,:), xx2(:,:)
  character(len=*), intent(in) :: fn

  if (ANY(shape(xx1) /= shape(xx2)))  then 
     write(errstr,*) fn//': array shapes not equal. Shape1: ',shape(xx1),' :Shape2 ',shape(xx2)
     call error ('histo')
  endif

 end subroutine compare2_SingleSingle

 !---------------------------------------------------------------------------
 subroutine compare1_SingleInt(v1, v2, fn)
  !---------------------------------------------------------------------------
  real(single),     intent(in) :: v1(:)
  integer,          intent(in) :: v2(:)
  character(len=*), intent(in) :: fn

  if (size(v1) /= size(v2))  then 
     write(errstr,*) fn//': vector sizes not equal. Size1: ',size(v1),' :Size2 ',size(v2)
     call error ('histo')
  endif

 end subroutine compare1_SingleInt

 !---------------------------------------------------------------------------
 subroutine compare2_SingleInt(xx1, xx2, fn)
  !---------------------------------------------------------------------------
  real(single),  intent(in)    :: xx1(:,:)
  integer,       intent(in)    :: xx2(:,:)
  character(len=*), intent(in) :: fn

  if (ANY(shape(xx1) /= shape(xx2)))  then 
     write(errstr,*) fn//': array shapes not equal. Shape1: ',shape(xx1),' :Shape2 ',shape(xx2)
     call error ('histo')
  endif

 end subroutine compare2_SingleInt


end module notNearTest
